/************************************/
/* Druhy projekt z PG               */
/* Transformace v 3D                */
/* Autor: Martin KOSTURIK     @1998 */
/* L95289                           */
/************************************/

#include <stdio.h>
#include <graphics.h>
#include <conio.h>
#include <math.h>
#include <stdlib.h>
#include <ctype.h>

#define ENTER 13
#define ESC	  27
#define RAD(x) (x * M_PI / 180)
#define MAXVRCH 8

int alfa = 30, beta = 30;
float otoceni = 10., posunuti = 10., zvetseni = 1.1;
float Jx, Jy, Jz;

float matvrch[MAXVRCH][3] =		  {  0. , 0. , 0. ,
									30. , 0. , 0. ,
									30. , 0. ,30. ,
									 0. , 0. ,30. ,
									 0. ,30. , 0. ,
									30. ,30. , 0. ,
									30. ,30. ,30. ,
									 0. ,30. ,30.   };

int matsous[MAXVRCH][MAXVRCH] =	  { 0,1,0,1,1,0,0,0,
									1,0,1,0,0,1,0,0,
									0,1,0,1,0,0,1,0,
									1,0,1,0,0,0,0,1,
									1,0,0,0,0,1,0,1,
									0,1,0,0,1,0,1,0,
									0,0,1,0,0,1,0,1,
									0,0,0,1,1,0,0,0  };

void init(void);
void osy(int mx, int my);
void p3to2d(float x, float y, float z, int *novex, int *novey);
void kresli(void);
void otoc3d(char osa, int uhel);
void posun3d(int osa, float dist);
void zvec3d(float zvec);
void vektory(void);


main()
{
	int mx, my, znak, konec;

	vektory();
	init();
	konec = 0;
	do {
		znak = getch();
		if(isupper(znak))
				znak = tolower(znak);
		switch(znak) {
			case 'x':
				otoc3d('x', otoceni);
				break;
			case 'y':
				otoc3d('y', otoceni);
				break;
			case 'z':
				otoc3d('z', otoceni);
				break;
			case 'w':
				posun3d(0, posunuti);
				break;
			case 's':
				posun3d(0, -posunuti);
				break;
			case 'h':
				posun3d(1, posunuti);
				break;
			case 'n':
				posun3d(1, -posunuti);
				break;
			case 'q':
				posun3d(2, posunuti);
				break;
			case 'a':
				posun3d(2, -posunuti);
				break;
			case 'v':
				zvec3d(zvetseni);
				break;
			case 'b':
				zvec3d(1. / zvetseni);
				break;
			case 'p':
				closegraph();
				init();
				break;
			case ESC:
				konec=1;
		}
	} while(!konec);

	closegraph();
	_setcursortype(_NORMALCURSOR);

return(0);
}


void osy(int mx, int my)
{
	setlinestyle(DOTTED_LINE, 1, NORM_WIDTH);
	line(mx, my , mx, 1);
	moveto(mx + 5, 2);
	outtext("z");
	line(mx, my, 1, my + mx * tan(RAD(alfa)));
	moveto(10, getmaxy() - 10);
	outtext("x");
	line(mx, my ,getmaxx(), my  + mx * tan(RAD(beta)));
	moveto(getmaxx() - 10, getmaxy() - 10);
	outtext("y");
	setlinestyle(SOLID_LINE, 1, NORM_WIDTH);
	outtextxy(mx - 100, getmaxy() - 10, "Klavesa 'p' = napoveda");
	setlinestyle(SOLID_LINE, 0, THICK_WIDTH);

return;
}


void p3to2d(float x,float y,float z,int *novex,int *novey)
{
	*novex = -Jx * cos(RAD(alfa)) * x + Jy * cos(RAD(beta)) * y + (getmaxx() / 2);
	*novey = Jx * sin(RAD(alfa)) * x + Jy * sin(RAD(beta)) * y - Jz * z + (getmaxy() / 2);

return;
}


void kresli(void)
{
	int prevod[MAXVRCH][2], i, j;

	for(i=0; i < MAXVRCH; i++)
		p3to2d(matvrch[i][0], matvrch[i][1], matvrch[i][2], &prevod[i][0], &prevod[i][1]);

	for(i=0; i<MAXVRCH; i++) {
		for(j=i; j<MAXVRCH; j++) {
			if(matsous[i][j])
				line(prevod[i][0], prevod[i][1], prevod[j][0], prevod[j][1]);
		}
	}
return;
}


void otoc3d(char osa, int uhel)
{
	int i;
	float p, uh;

	kresli();

	uh = RAD(uhel);
	for(i=0; i < MAXVRCH; i++) {
		switch(osa) {
			case 'x':
				p = matvrch[i][1];
				matvrch[i][1] = p * cos(uh) - matvrch[i][2] * sin(uh);
				matvrch[i][2] = p * sin(uh) + matvrch[i][2] * cos(uh);
				break;
			case 'y':
				p = matvrch[i][0];
				matvrch[i][0] = p * cos(uh) + matvrch[i][2] * sin(uh);
				matvrch[i][2] = -p * sin(uh) + matvrch[i][2] * cos(uh);
				break;
			case 'z':
				p = matvrch[i][0];
				matvrch[i][0] = p * cos(uh) - matvrch[i][1] * sin(uh);
				matvrch[i][1] = p * sin(uh) + matvrch[i][1] * cos(uh);
				break;
		}
	}
	kresli();

return;
}


void posun3d(int osa, float dist)
{
	int i;

	kresli();

	for(i=0; i < MAXVRCH; i++)
		matvrch[i][osa] = matvrch[i][osa] + dist;

	kresli();

return;
}


void zvec3d(float zvec)
{
	int i;

	kresli();

	for (i=0; i < MAXVRCH; i++) {
		matvrch[i][0] = matvrch[i][0] * zvec;
		matvrch[i][1] = matvrch[i][1] * zvec;
		matvrch[i][2] = matvrch[i][2] * zvec;
	}
	kresli();

return;
}


void init(void)
{
	int znak, alfadeg, betadeg;
	int graph_driver = DETECT, graph_mode = VGAHI, grchyba;

	_setcursortype(_NOCURSOR);
	textbackground(BLACK);
	clrscr();
	textcolor(YELLOW);
	cprintf("%30sTRANSFORMACE V 3D\r\n", " ");
	textcolor(RED);
	cprintf("%23s@1998 KOSTURIK Martin (L95289)\r\n", " ");
	cprintf("%23sDruhy semestralni projekt z PG\r\n\n", " ");
	textcolor(YELLOW);
	cprintf("OVLADANI :\r\n\n");
	textcolor(WHITE);
	cprintf("Otoceni  v ose x = X\r\n");
	cprintf("Otoceni  v ose y = Y\r\n");
	cprintf("Otoceni  v ose z = Z\r\n\n");
	cprintf("Posunuti v ose x = W(+) S(-)\r\n");
	cprintf("Posunuti v ose y = H(+) N(-)\r\n");
	cprintf("Posunuti v ose z = Q(+) A(-)\r\n\n");
	cprintf("Zmena meritka    = V(+) B(-)\r\n\n");
	cprintf("Zmena axonomet.  = I\r\n");
	cprintf("Zmena transfor.  = O\r\n\n");
	cprintf("Tato napoveda    = P\r\n");
	cprintf("KONEC            = ESC\r\n\n");
	textcolor(YELLOW);
	gotoxy(40, 5);
	cprintf("NASTAVENI UHLU AXONOMETRIE :");
	gotoxy(40, 14);
	cprintf("O KOLIK ...");
	gotoxy(25, 25);
	cprintf("Stiskni ENTER pro vykresleni");

	while(1) {
		textcolor(WHITE);
		gotoxy(40, 7);
		cprintf("Uhel ALFA pro osu x (45 stupnu): %3d", alfa);
		gotoxy(40,8);
		cprintf("Uhel BETA pro osu y (0 stupnu) : %3d", beta);
		gotoxy(40,10);
		cprintf("Jednotkovy vektor Jx = %2.5f", Jx);
		gotoxy(40,11);
		cprintf("Jednotkovy vektor Jy = %2.5f", Jy);
		gotoxy(40,12);
		cprintf("Jednotkovy vektor Jz = %2.5f", Jz);
		gotoxy(40, 16);
		cprintf("otacet    ? (10) : %2.2f", otoceni);
		gotoxy(40, 17);
		cprintf("posouvat  ? (10) : %2.2f", posunuti);
		gotoxy(40, 18);
		cprintf("zvetsovat ? (1.1): %2.2f", zvetseni);

		znak = getch();
		if(isupper(znak))
			znak = tolower(znak);
		if(znak == ESC) {
			clrscr();
			exit(0);
		}
		if(znak == ENTER)	break;
		if(znak == 'i') {
			_setcursortype(_NORMALCURSOR);
			gotoxy(73, 7);
			cprintf("   \b\b\b");
			cscanf("%3d", &alfa);
			gotoxy(73, 8);
			cprintf("   \b\b\b");
			cscanf("%3d", &beta);
			vektory();
			_setcursortype(_NOCURSOR);
			getch();
		}
		if(znak == 'o') {
			_setcursortype(_NORMALCURSOR);
			gotoxy(59, 16);
			cprintf("     \b\b\b\b\b");
			cscanf("%5f", &otoceni);
			gotoxy(59, 17);
			cprintf("     \b\b\b\b\b");
			cscanf("%5f", &posunuti);
			gotoxy(59, 18);
			cprintf("     \b\b\b\b\b");
			cscanf("%5f", &zvetseni);
			_setcursortype(_NOCURSOR);
			getch();
		}
	}
	grchyba = registerbgidriver(EGAVGA_driver);
	if(grchyba < 0) {
		printf("\nChyba grafiky: %s", grapherrormsg(grchyba));
		exit(1);
	}
	initgraph(&graph_driver, &graph_mode, "");

	cleardevice();
	setbkcolor(LIGHTGRAY);
	setcolor(RED);
	setlinestyle(SOLID_LINE, 0, THICK_WIDTH);
	moveto(0, 0);
	lineto(getmaxx(), 0);
	lineto(getmaxx(), getmaxy());
	lineto(0, getmaxy());
	lineto(0, 0);
	setcolor(BLUE);

	settextstyle(SMALL_FONT, HORIZ_DIR, 4);
	osy(getmaxx() / 2, getmaxy() / 2);
	setwritemode(1);

	kresli();

return;
}

void vektory(void)
{
	float a = RAD(alfa),
		  b = RAD(beta);

	if ( (cos(a) == 0) || (beta == 0) ) Jx = 0 ;
	else Jx = 1 / ( cos(a) * sqrt( 1 + sin(a) * cos(b) / cos(a) / sin(b)));

	if ( (cos(b) == 0) || (alfa == 0) ) Jy = 0 ;
	else Jy = 1 / ( cos(b) * sqrt( 1 + sin(b) * cos(a) / cos(b) / sin(a)));

	if ( (cos(a) == 0) || (cos(a + b) <= 1e-6) ) Jz = 0 ;
	else Jz = 1 / ( cos(a) * sqrt( 1 + sin(a) * sin(a + b) / cos(a) / cos(a + b)));

return;
}